/***********************************************************
ROBUST.PRG

This is a GAUSS program, written by
Bruce E. Hansen
Department of Economics
University of Wisconsin
bhansen@ssc.wisc.edu
http:/www.ssc.wisc.edu/~bhansen

This program estimates the spline transfer function model presented in
"How Responsive are Private Transfers to Income?
Evidence From a Laissez-Faire Economy"
by Donald Cox, Bruce Hansen, and Emmanuel Jimenez.

The results of this program generates the estimates reported in Table 6.

This program will take a long time to execute if one sets qn=0.  To speed up
execution (with little cost in precision), set qn=500 or even qn=100.

***********************************************************/

trm = .02;            @ Percentile of high-income households deleted  @
_per = .10;           @ Percentage to trim from quantiles for threshold search @
qn   = 0;             @ Set qn=0  to search across all thresholds.
                        Set qn to a positive integers to search
                        across a fixed number of quantiles     @
_noinc=1;             @ If set to 1, includes dummy for no income, else set to 0 @

/***********************************************************/
output file = robust.out reset;
output off;
/***********************************************************/

model=1; do while model<=9;

smp=1; do while smp<=2;              @ Set to 1 if "urban" or 2 if "rural"  @

output on;
"************************************************************";
"";"";
_noinc=1;
if model==1;
  "International Transfer Recipients Excluded";
  if smp==1; _noinc=1; else; _noinc=0; endif;
elseif model==2;
  "Not Secondary Graduate";
elseif model==3;
  "Secondary Graduate";
elseif model==4;
  "Family Size 4 or Less";
elseif model==5;
  "Family Size 5 or More";
  if smp==1; _noinc=1; else; _noinc=0; endif;
elseif model==6;
  "Household Head Aged 45 or less";
  if smp==1; _noinc=1; else; _noinc=0; endif;
elseif model==7;
  "Household Head over age 45";
  if smp==1; _noinc=1; else; _noinc=0; endif;
elseif model==8;
  "Gross Transfers Received";
elseif model==9;
  "Gross Transfers Given (Gifts)";
endif;
output off;


let rname =
"Region1"
"Region2"
"Region3"
"Region4"
"Region5"
"Region6"
"Region7"
"Region8"
"Region9"
"Region10"
"Region11"
"Region12"
"Region13";
if smp==1;
  rname ="Region0"|rname;
  sample="Urban";
elseif smp==2;
  sample="Rural";
endif;

dfile = "" $+ sample $+ ".dat";
vn = getname(dfile);
open f1=^dfile;
dat=readr(f1,20000);
f1=close(f1);

abroad=dat[.,1];
domestic=dat[.,2];
inkind=dat[.,3];
gifts=dat[.,4];
totinc=dat[.,5];
retinc=dat[.,10];
yname = "Transfers";
i = totinc+gifts-abroad-domestic-inkind-retinc;
ii=sortc(i,1);
qi=ii[floor(rows(dat)*(1-trm))];

dd=(i.<0)+(i.>qi);
if model==1;
  dd=dd+(1-dd).*(abroad.>0);
elseif model==2;
  dd=dd+(1-dd).*(dat[.,14]+dat[.,15]+dat[.,16].>0);
elseif model==3;
  dd=dd+(1-dd).*(dat[.,12]+dat[.,13].>0);
elseif model==4;
  dd=dd+(1-dd).*(dat[.,23].>4);
elseif model==5;
  dd=dd+(1-dd).*(dat[.,23].<=4);
elseif model==6;
  dd=dd+(1-dd).*(dat[.,17].>45);
elseif model==7;
  dd=dd+(1-dd).*(dat[.,17].<=45);
endif;

if model==1;
  y = domestic+inkind-gifts;
elseif model==8;
  y = abroad+domestic+inkind;
elseif model==9;
  y = gifts;
else;
  y = abroad+domestic+inkind-gifts;
endif;

dat=delif(dat~i~y,dd);
n=rows(dat);
retinc=dat[.,10];
i=dat[.,28];
y=dat[.,29];

dat[.,23]=dat[.,23]-dat[.,20]-dat[.,21]-dat[.,22];
vn[23]="Adults";

if model==2;
  xx=(retinc.>0)~retinc~dat[.,17];
  xn="Has Retr"|vn[10 17];
elseif model==3;
  xx=(retinc.>0)~retinc~dat[.,17];
  xn="Has Retr"|vn[10 17];
else;
  xx=(retinc.>0)~retinc~dat[.,12:17];
  xn="Has Retr"|vn[10 12:17];
endif;

region=dat[.,27];
r=unique(region,1);
xreg=(region.==(r'));
xx=xx~dat[.,18:26]~xreg;
xn=xn|vn[18:26]|rname;
mx = ones(n,1)*(meanc(xx)');
if _noinc==1;
  i0=(i.==0);
  xx=i0~xx;
  xn="No Income"|xn;
endif;

/*************************************************/
@ Linear Spline Model @

xm = xx*invpd(moment(xx,0));
e0 = y - xm*(xx'y);
ee0 = e0'e0;
q=sortc(i,1);
if qn == 0;
  q = unique(q[floor(_per*n):n-floor(_per*n)],1);
  qn = rows(q);
else;
  q = q[floor(seqa(_per,(1-_per*2)/qn,qn)*n)];
endif;

sse = zeros(qn,1);
j=1;do while j<=qn;
  qj = q[j];
  ic = i - qj;
  i1 = (i.<=qj);
  i2 = 1-i1;
  dx = (ic.*i1)~(ic.*i2);
  xv = dx - xm*(xx'dx);
  et = e0 - xv*(e0/xv);
  sse[j] = et'et;
  if round(j/10)==(j/10);
    j~qj~(sse[j]/n);
  endif;
j=j+1;endo;
jmin = minindc(sse);
knot = q[jmin];

ic = i - knot;
i1 = (i.<=knot);
i2 = 1-i1;
dx = (ic.*i1)~(ic.*i2);
x  = dx~xx;
beta = y/x;
e = y - x*beta;
xh = -beta[1].*i1-beta[2].*i2;
x=x~xh;
tr1 = dx*beta[1:2]+mx*beta[3+_noinc:rows(beta)];

xname = "Slope_1"|"Slope_2";
xname=xname|xn|"Threshold";
theta=beta|knot;

output on;
"";"";
sh = sample $+ " households";
$sh;
"Observations                      " n;
"__________________________________________________";
  ee = e'e;
  sig=ee/(rows(y)-cols(x));
  mi = invpd(moment(x,0));
  se = sqrt(abs(diag(mi*sig)));
  seh=sqrt(abs(diag(mi*moment(x.*e,0)*mi)));
  xse=xname~theta~se~seh;
  if _noinc==0;
    xse=xse[1 2,.]|xse[rows(xse),.];
  else;
    xse=xse[1 2 3,.]|xse[rows(xse),.];
  endif;
  ff = "#*.*lG";
  pr = printfm("Variable"~"Estimate"~"St Error"~"Het St Er",0~0~0~0,ff~16~8);"";
  "       ____________________________________________________";
  pr = printfm(xse,0~1~1~1,ff~16~8);"";
  "__________________________________________________";
  "";"";"";
output off;

smp = smp+1; endo;

model=model+1;endo;